{-|
Module      : Data.Graph.Clustering.Louvain.CplusPlus
Description : C++ implementation of louvain
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : alexandre.delanoe+louvain@iscpif.fr
Stability   : experimental
Portability : POSIX

git clone https://gitlab.iscpif.fr/gargantext/clustering-louvain-cplusplus
-}

{-# LANGUAGE OverloadedStrings #-}

module Data.Graph.Clustering.Louvain.CplusPlus
  where

import qualified Data.List as L
import Data.Map.Strict (Map, toList)
import Data.Monoid ((<>))

import Data.Maybe (maybe)

import qualified Control.Foldl as Fold

import qualified Turtle as TU
import System.IO

import Data.Text (Text)
import qualified Data.Text    as T
import qualified Data.Text.IO as T

data LouvainNode = LouvainNode { l_node_id :: Int
                               , l_community_id :: Int
                               } deriving (Show)


cLouvain :: Map (Int, Int) Double -> IO [LouvainNode]
cLouvain ms = do
  let inFileD = "/tmp/louvainData.txt"
  let inFileW = "/tmp/louvainWeights.weights"
  let outBin  = "/tmp/louvainGraph.bin"
  let outRes  = "/tmp/louvainGraph.tree"
  
  let louvain = "/usr/share/louvain/louvain"
  
  writeInput inFileD ms
  let cmdLouvain = louvain <> " " <> inFileD <> " " <> inFileW <> " " <> outBin <> " " <> outRes
  --pure cmdLouvain
  shell (T.pack cmdLouvain)
  myResult <- readOutput outRes
  pure myResult


writeInput :: FilePath -> Map (Int, Int) Double -> IO ()
writeInput fp ns = writeFile fp (unlines ns')
  where
    ns' = map (\((n1,n2), _) -> show n1 <> " " <> show n2) (toList ns)
    --ns' = map (\((n1,n2), w) -> show n1 <> " " <> show n2 <> " " <> show w) (toList ns)

louvainNode :: [Text] -> LouvainNode
louvainNode [n1,n2] = LouvainNode (read $ T.unpack n1) (read $ T.unpack n2)
louvainNode _       = error "Louvain node not found"

readOutput :: FilePath -> IO [LouvainNode]
readOutput fp = do
  myData <- T.readFile fp
  pure $  map (\s -> louvainNode $ T.splitOn " " s) $ L.tail $ T.lines myData

shell :: TU.MonadIO io => Text -> io TU.ExitCode
shell cmd = do
  x <- TU.shell cmd TU.empty
  case x of
      TU.ExitSuccess   -> return TU.ExitSuccess
      TU.ExitFailure n -> TU.die (cmd <> " failed with exit code: " <> TU.repr n)

